﻿using Microsoft.AspNetCore.Mvc;
using ExperienceEfCore.Models;
using ExperienceEfCore.Services;
using ExperienceEfCore.Data;
using System.Runtime.CompilerServices;
using Microsoft.AspNetCore.Authorization;


namespace ExperienceEfCore.Controllers;

[Authorize]
[ApiController]
[Route("[controller]")]
public class GuestController : ControllerBase
{
    private MyDbContext _context;

    public GuestController(MyDbContext context)
    {
        _context = context;
    }
    
    [Authorize(Policy = "ManagerOnly")]
    [HttpGet("GuestNameBySharedExperience")]
    public ActionResult<List<string>> GetGuestNamesBySharedExperienceId(int sharedExperienceId)
    {
        var guestNames = GuestServices.GetGuestNamesBySharedExperienceId(_context, sharedExperienceId);
        if (guestNames == null || guestNames.Count == 0)
        {
            return NotFound("No guest names found for the given shared experience ID.");
        }
        return Ok(guestNames); // 200 OK with the list of guest names
    }

    [Authorize(Policy = "ManagerOnly")]
    [HttpGet("GuestNamesForExperienceAndSharedExperience")]
    public ActionResult<List<string>> GetGuestNamesForExperienceAndSharedExperience(int sharedExperienceId, int experienceId)
    {
        var guestNames = GuestServices.GetGuestNamesForExperienceAndSharedExperience(_context, sharedExperienceId, experienceId);
        if (guestNames == null || guestNames.Count == 0)
        {
            return NotFound("No guest names found for the given shared experience and experience IDs.");
        }
        return Ok(guestNames); // 200 OK with the list of guest names
    }


}
