using ExperienceEfCore.Attributes;
using Microsoft.AspNetCore.Mvc;
using ExperienceEfCore.Models;
using ExperienceEfCore.Services;
using ExperienceEfCore.Data;
using Microsoft.AspNetCore.Identity;
using Microsoft.AspNetCore.Authorization;
using System.Security.Claims;

namespace ExperienceEfCore.Controllers;

[Authorize(Policy = "AdminOnly")]
[ApiController]
[Route("[controller]")]
public class SystemController : ControllerBase
{
    private MyDbContext _context;
     private readonly UserManager<ApiUser> _userManager;

    public SystemController(MyDbContext context, UserManager<ApiUser> userManager){
        _context = context;
        _userManager = userManager;
    }

    [HttpPost("SeedDatabase")]
    public ActionResult SeedDatabase()
    {
        try
        {
            SystemService.SeedDatabase(_context); // Seed the database
            return Ok("Database seeded successfully."); // Return success message with HTTP 200
        }
        catch (Exception ex)
        {
            // If an error occurs during the seeding process, return a BadRequest with the error message
            return BadRequest($"An error occurred while seeding the database: {ex.Message}");
        }
    }

    [HttpPut(Name = "SeedUsers")]
    public async Task<IActionResult> Put()
        {
            var seededUsers = 0;

            var adminUser = new ApiUser
            {
                UserName = "Admin",
                Email = "admin@api.com",
                FullName = "Admin Adminsen"
            };

            if (_userManager.FindByEmailAsync(adminUser.Email).Result == null)
            {
                var userResult = _userManager.CreateAsync(adminUser, "AdminExperience1!").Result;

                if (!userResult.Succeeded)
                    throw new Exception(userResult.ToString());

                //Add superUser  Claim
                var roleResult = _userManager.AddClaimAsync(adminUser, new Claim("IsAdmin", "true")).Result;

                if (!roleResult.Succeeded)
                    throw new Exception(roleResult.ToString());

                seededUsers++;
            }
        
        return new JsonResult(new {SeededUsers = seededUsers});
        }
}
