﻿using ExperienceEfCore.Data;
using Microsoft.EntityFrameworkCore;

namespace ExperienceEfCore.Services
{
    public static class GuestServices
    {
        public static class GuestService
        {
            public static List<string?> GetAllGuestNames(MyDbContext context)
            {
                return context.Guests
                    .Select(g => g.Name)
                    .ToList();
            }
        }
        // Guests registered for shared experience
        public static List<string> GetGuestNamesBySharedExperienceId(MyDbContext context, int sharedExperienceId)
        {
            var guestNames = context.Guests
                .Where(g => g.SharedExperiences.Any(se => se.Id == sharedExperienceId))
                .Select(g => g.Name)
                .Distinct()
                .ToList();

            return guestNames;
        }
        // Get the guests registered for one of the experiences in a shared experience
        public static List<string> GetGuestNamesForExperienceAndSharedExperience(MyDbContext context, int sharedExperienceId, int experienceId)
        {
            var result = context.Guests
            .Where(g => g.Experiences.Any(e => e.Id == experienceId) &&
                        g.SharedExperiences.Any(se => se.Id == sharedExperienceId))
            .Select(g => g.Name)
            .Distinct()
            .ToList();

            return result;
        }
    }
}
