--CREATION OF DATABASE, TABLES AND RELATIONSHIPS
/*
CREATE DATABASE Experience

CREATE TABLE Provider (
    businessAddress VARCHAR(120) NULL,
    PhoneNo VARCHAR(20) NULL,
    CVR INT NOT NULL,
    PRIMARY KEY CLUSTERED (CVR ASC)
);

CREATE TABLE Experience (
    id INT NOT NULL,
    title VARCHAR(40) NULL,
    price DECIMAL(18, 2) NULL,
    PRIMARY KEY CLUSTERED (id ASC)
);

CREATE TABLE Guest (
    id INT NOT NULL,
    name VARCHAR (20) NULL,
    age INT NULL,
    phoneNo VARCHAR (20) NULL,
    PRIMARY KEY CLUSTERED (id ASC)
);

CREATE TABLE SharedExperience (
    id INT NOT NULL,
    title VARCHAR (30)  NULL,
    descriptionSharedExperience VARCHAR(100) NULL,
    dateSharedExperience VARCHAR(120) NULL,
    PRIMARY KEY CLUSTERED (id ASC)
);
--Junction table
CREATE TABLE RegisteredGuestSharedExperience (
    SharedExperienceID INT NOT NULL,
    GuestID INT NOT NULL,
    PRIMARY KEY (SharedExperienceID, GuestID),
    FOREIGN KEY (SharedExperienceID) REFERENCES SharedExperience(id) ON DELETE CASCADE,
    FOREIGN KEY (GuestID) REFERENCES Guest(id) ON DELETE CASCADE
);

--Junction table
CREATE TABLE RegisteredGuestExperience (
    ExperienceID INT NOT NULL,
    GuestID INT NOT NULL,
    PRIMARY KEY (ExperienceID, GuestID),
    FOREIGN KEY (ExperienceID) REFERENCES Experience(id) ON DELETE CASCADE,
    FOREIGN KEY (GuestID) REFERENCES Guest(id) ON DELETE CASCADE
);
--Junction table
CREATE TABLE ExperienceList (
    ExperienceID INT NOT NULL,
    SharedExperienceID INT NOT NULL,
    PRIMARY KEY (ExperienceID, SharedExperienceID),
    FOREIGN KEY (ExperienceID) REFERENCES Experience(id) ON DELETE CASCADE,
    FOREIGN KEY (SharedExperienceID) REFERENCES SharedExperience(id) ON DELETE CASCADE
);
*/

/* -----------------INSERTION OF DUMMY DATA--------------------------
INSERT INTO Provider (CVR, PhoneNo, businessAddress) VALUES
(11111114, +45 71555080, 'Finlandsgade 17, 8200 Aarhus N')

INSERT INTO Experience (id, title, price) VALUES
(1,'Night at Noahs Hotel Single room', 730.50),
(2,'Night at Noahs Hotel Double room', 910.99),
(3,'Flight AAR - VIE', 1000.70),
(4,'Vienna Historic Center Walking Tour', 100.00)

INSERT INTO SharedExperience (id, title, descriptionSharedExperience, dateSharedExperience) VALUES
(1,'Trip to Austria', 'Trip to Austria with great food and views', 2024-07-02),
(2,'Dinner downtown', 'Dinner at Burger King, followed by danceclub', 2024-04-07),
(3,'Pottery Weekend', 'Create beatiful pottery, and learn to focus for a whole weekend', 2024-03-22)

INSERT INTO Guest (id, name, age, phoneNo) VALUES
(1, 'Joan', 23, '+45 12435687'),
(2, 'Suzanne', 32, '+45 87654321'),
(3, 'Patrick', 13, '+45 10000001'),
(4, 'Anne', 83, '+45 12344321')

INSERT INTO RegisteredGuestSharedExperience (SharedExperienceID, GuestID) VALUES
(1,1), //Joan
(1,2), //Suzanne
(1,3), //Patrick
(1,4) //Anne

INSERT INTO ExperienceList (ExperienceID, SharedExperienceID) VALUES
(3,1), //Flight
(1,1), //Single room
(4,1) //Walking tour

INSERT INTO RegisteredGuestExperience (ExperienceID, GuestID) VALUES
(4,1), --Walking tour, Joan
(4,2) --Walking tour, Suzanne
(1,1), --Noah's hotel single room, Joan
(1,2), --Noah's hotel single room, Suzanne
(1,3), --Noah's hotel single room, Patrick
(1,4) --Noah's hotel single room, Anne

INSERT INTO RegisteredGuestExperience (ExperienceID, GuestID) VALUES
(3,1), --Flight, Joan
(3,2), --Flight, Suzanne
(3,3), --Flight, Patrick
(3,4)  --Flight, Anne
*/


/* -------------GET/SELECT THE MINIMAL QUERIES-----------------*/
--Experience provider data
SELECT * FROM Provider

--Available experience data
-- (ExperienceServices)
SELECT * FROM Experience

--Shared experience data (descending by date)
-- (SharedExperienceServices)
SELECT * FROM SharedExperience ORDER BY dateSharedExperience desc


--Guest registered for shared experience
-- (SystemServices)
SELECT DISTINCT G.name
FROM Guest G
    INNER JOIN RegisteredGuestExperience RGE ON G.id = RGE.GuestID
    INNER JOIN Experience E ON RGE.ExperienceID = E.id
    INNER JOIN ExperienceList EL ON E.id = EL.ExperienceID  -- Join Experience with ExperienceList
    INNER JOIN RegisteredGuestSharedExperience RGSE ON EL.SharedExperienceID = RGSE.SharedExperienceID  -- Join ExperienceList with RegisteredGuestSharedExperience
WHERE RGSE.SharedExperienceID = 1;  -- Filtering by a valid SharedExperienceID (1, 2, or 3)

-- Get the experiences included in a shared experience
-- (SystemServices)
SELECT 
    SE.title AS SharedExperienceTitle,
    E.title AS ExperienceTitle
FROM 
    SharedExperience SE
    INNER JOIN ExperienceList EL ON SE.id = EL.SharedExperienceID
    INNER JOIN Experience E ON EL.ExperienceID = E.id
WHERE 
    SE.id = 1;  -- Replace with the desired SharedExperienceID


-- Get the guests registered for one of the experiences in a shared experience
-- (SystemServices)
SELECT 
    E.title AS ExperienceTitle,
    G.name AS GuestName
FROM 
    Experience E
    INNER JOIN RegisteredGuestExperience RGE ON E.id = RGE.ExperienceID
    INNER JOIN Guest G ON RGE.GuestID = G.id
    INNER JOIN ExperienceList EL ON E.id = EL.ExperienceID
    INNER JOIN SharedExperience SE ON EL.SharedExperienceID = SE.id
WHERE 
    SE.id = 1 
    AND E.id = 4;



--Min, AVG, Max
-- (ExperienceServices)
SELECT 
    MIN(price) AS min_price,
    AVG(price) AS avg_price,
    MAX(price) AS max_price
FROM Experience;

-- Title, numberOfGuests, priceSum
-- (SystemServices)
SELECT 
    e.Title AS Experiences,
    COUNT(j.ExperienceID) AS Amount,
    COUNT(j.ExperienceID) * e.Price AS Price
FROM Experience e
LEFT JOIN RegisteredGuestExperience j ON e.id = j.ExperienceID  -- Join on ID, not Title
GROUP BY e.Title, e.Price;

-- Additional query
-- (SystemServices)
SELECT Experience.Title, 
       CASE 
           WHEN COUNT(RegisteredGuestExperience.GuestID) >= 50 THEN '20% discount'
           WHEN COUNT(RegisteredGuestExperience.GuestID) >= 10 THEN '10% discount'
           ELSE 'No discount'
       END AS Discount
FROM Experience 
LEFT JOIN RegisteredGuestExperience ON Experience.id = RegisteredGuestExperience.ExperienceID 
GROUP BY Experience.Title;

